import unittest
import sys
import os

from utils import execute, kill, run_file, run_barebone

def flatten(lst):
    new_lst = []
    for f in lst:
        if isinstance(f, (list, tuple)):
            new_lst.extend(flatten(f))
        else:
            new_lst.append(f)
    return new_lst

bottom = [
        '"model"',
        '"instantiate_bottom"', 1,
        '"add_node"', 1, '"Class"', 
        '"add_node"', 1, '"Type"',
        '"add_node"', 1, '"Any"',
        '"add_node"', 1, '"String"',
        '"add_value"', 1, '"inheritance"', '"inheritance"',
        '"add_value"', 1, '"link"', '"link"',
        '"add_value"', 1, '"name"', '"name"',
        '"add_edge"', 1, '"l1"', '"Class"', '"Any"',
        '"add_edge"', 1, '"l2"', '"Type"', '"Any"',
        '"add_edge"', 1, '"Inheritance"', '"Any"', '"Any"',
        '"add_edge"', 1, '"l4"', '"Inheritance"', '"inheritance"',
        '"add_edge"', 1, '"Association"', '"Any"', '"Any"',
        '"add_edge"', 1, '"l6"', '"Association"', '"Any"',
        '"add_edge"', 1, '"l7"', '"Association"', '"link"',
        '"add_edge"', 1, '"l8"', '"Association"', '"String"',
        '"add_edge"', 1, '"l9"', '"l8"', '"name"',
        '"exit"',
    ]

retype = [
        '"model"', 
        '"retype_model"', 1, 1,
        '"define_inheritance"', 1, '"Inheritance"',
        '"retype"', 1, '"Class"', '"Class"',
        '"retype"', 1, '"Type"', '"Class"',
        '"retype"', 1, '"Any"', '"Class"',
        '"retype"', 1, '"String"', '"Type"',
        '"retype"', 1, '"inheritance"', '"String"',
        '"retype"', 1, '"link"', '"String"',
        '"retype"', 1, '"name"', '"String"',
        '"retype"', 1, '"l1"', '"Inheritance"',
        '"retype"', 1, '"l2"', '"Inheritance"',
        '"retype"', 1, '"Inheritance"', '"Association"',
        '"retype"', 1, '"l4"', '"l8"',
        '"retype"', 1, '"Association"', '"Association"',
        '"retype"', 1, '"l6"', '"Inheritance"',
        '"retype"', 1, '"l7"', '"l8"',
        '"retype"', 1, '"l8"', '"Association"',
        '"retype"', 1, '"l9"', '"l8"',
        '"exit"',
    ]

instantiate_scd = [
        '"model"',
        '"instantiate_model"', 1, 2,
        '"define_inheritance"', 2, '"Inheritance"',
        '"instantiate_node"', 2, '"Class"', '"Place"',
        '"instantiate_node"', 2, '"Class"', '"Transition"',
        '"instantiate_node"', 2, '"Type"', '"Integer"',
        '"instantiate_link"', 2, '"Association"', '"P2T"', '"Place"', '"Transition"',
        '"instantiate_link"', 2, '"Association"', '"T2P"', '"Transition"', '"Place"',
        '"instantiate_named"', 2, '"Association"', '"tokens"', '"Place"', '"Integer"',
        '"instantiate_named"', 2, '"Association"', '"weight"', '"P2T"', '"Integer"',
        '"instantiate_named"', 2, '"Association"', '"weight"', '"T2P"', '"Integer"',
        '"exit"',
    ]

instantiate_pn = [
        '"model"',
        '"instantiate_model"', 2, 3,
        '"instantiate_node"', 3, '"Place"', '"p1"',
        '"instantiate_node"', 3, '"Place"', '"p2"',
        '"instantiate_node"', 3, '"Transition"', '"t1"',
        '"instantiate_node"', 3, '"Transition"', '"t2"',
        '"instantiate_link"', 3, '"P2T"', '"p1_t1"', '"p1"', '"t1"',
        '"instantiate_link"', 3, '"T2P"', '"t1_p2"', '"t1"', '"p2"',
        '"instantiate_link"', 3, '"P2T"', '"p2_t2"', '"p2"', '"t2"',
        '"instantiate_link"', 3, '"T2P"', '"t2_p1"', '"t2"', '"p1"',
        '"instantiate_attribute"', 3, '"p1_t1"', '"weight"', '1',
        '"instantiate_attribute"', 3, '"t1_p2"', '"weight"', '2',
        '"instantiate_attribute"', 3, '"p2_t2"', '"weight"', '3',
        '"instantiate_attribute"', 3, '"t2_p1"', '"weight"', '4',
        '"instantiate_attribute"', 3, '"p1"', '"tokens"', '5',
        '"instantiate_attribute"', 3, '"p2"', '"tokens"', '6',
        '"exit"',
    ]

instantiate_example = [
        '"model"',
        '"instantiate_model"', 1, 2,
        '"define_inheritance"', 2, '"Inheritance"',
        '"instantiate_node"', 2, '"Class"', '"A"',
        '"instantiate_node"', 2, '"Class"', '"B"',
        '"instantiate_node"', 2, '"Class"', '"C"',
        '"instantiate_link"', 2, '"Inheritance"', '"b_inherits_a"', '"B"', '"A"',
        '"instantiate_named"', 2, '"Association"', '"tokens"', '"A"', '"B"',
        '"instantiate_named"', 2, '"Association"', '"tokens"', '"C"', '"B"',
        '"exit"',
        '"model"',
        '"instantiate_model"', 2, 3,
        '"instantiate_node"', 3, '"A"', '"a"',
        '"instantiate_attribute"', 3, '"a"', '"tokens"', '"b"',
        '"instantiate_node"', 3, '"B"', '"b"',
        '"instantiate_node"', 3, '"C"', '"c"',
        '"instantiate_attribute"', 3, '"c"', '"tokens"', '"b"',
        '"exit"',
    ]

def conformance_call(operation, model, metamodel):
    return [
            '"output"',
                '"call"',
                    '"access"', '"resolve"', '"%s"' % operation,
                    '3',
                    '"const"', 3,
                    '"call"',
                        '"access"', '"resolve"', '"dict_read"',
                            '2',
                            '"call"',
                                '"access"', '"resolve"', '"dict_read"',
                                    '2',
                                    '"const"', 3,
                                    '"const"', '"model"',
                                    'false',
                            '"const"', '"%s"' % model,
                        'false',
                    '"call"',
                        '"access"', '"resolve"', '"dict_read"',
                            '2',
                            '"call"',
                                '"access"', '"resolve"', '"dict_read"',
                                    '2',
                                    '"const"', 2,
                                    '"const"', '"model"',
                                    'false',
                            '"const"', '"%s"' % metamodel,
                        'false',
                'false',
            'true',
            ]

is_direct_instance = \
    conformance_call("is_direct_instance", "a", "A") + \
    conformance_call("is_direct_instance", "b", "A") + \
    conformance_call("is_direct_instance", "c", "A") + \
    conformance_call("is_direct_instance", "a", "B") + \
    conformance_call("is_direct_instance", "b", "B") + \
    conformance_call("is_direct_instance", "c", "B") + \
    conformance_call("is_direct_instance", "a", "C") + \
    conformance_call("is_direct_instance", "b", "C") + \
    conformance_call("is_direct_instance", "c", "C")

is_nominal_instance = \
    conformance_call("is_nominal_instance", "a", "A") + \
    conformance_call("is_nominal_instance", "b", "A") + \
    conformance_call("is_nominal_instance", "c", "A") + \
    conformance_call("is_nominal_instance", "a", "B") + \
    conformance_call("is_nominal_instance", "b", "B") + \
    conformance_call("is_nominal_instance", "c", "B") + \
    conformance_call("is_nominal_instance", "a", "C") + \
    conformance_call("is_nominal_instance", "b", "C") + \
    conformance_call("is_nominal_instance", "c", "C")

is_structural_instance = \
    conformance_call("is_structural_instance", "a", "A") + \
    conformance_call("is_structural_instance", "b", "A") + \
    conformance_call("is_structural_instance", "c", "A") + \
    conformance_call("is_structural_instance", "a", "B") + \
    conformance_call("is_structural_instance", "b", "B") + \
    conformance_call("is_structural_instance", "c", "B") + \
    conformance_call("is_structural_instance", "a", "C") + \
    conformance_call("is_structural_instance", "b", "C") + \
    conformance_call("is_structural_instance", "c", "C")

def conformance_check(node):
    return [
            '"output"',
                '"call"',
                    '"access"', '"resolve"', '"conformance_scd"',
                    '1',
                    '"const"', node,
                    'false',
                'true',
            ]

class TestConstructorsModels(unittest.TestCase):
    def test_constructors_instantiate_bottom(self):
        commands = bottom + retype + conformance_check(1) + ['"return"', 'false']
        self.assertTrue(run_barebone(commands, ["OK"], 1))

    def test_constructors_instantiate_scd(self):
        commands = bottom + retype + instantiate_scd + conformance_check(2) + ['"return"', 'false']
        self.assertTrue(run_barebone(commands, ["OK"], 1))

    def test_constructors_instantiate_pn(self):
        commands = bottom + retype + instantiate_scd + instantiate_pn + conformance_check(3) + ['"return"', 'false']
        self.assertTrue(run_barebone(commands, ["OK"], 1))

    def test_constructors_instantiate_example(self):
        commands = bottom + retype + instantiate_example + conformance_check(2) + conformance_check(3) + ['"return"', 'false']
        self.assertTrue(run_barebone(commands, ["OK", "OK"], 1))

    def test_constructors_is_direct_instance(self):
        commands = bottom + retype + instantiate_example + is_direct_instance + ['"return"', 'false']
        expected = ['True', 'False', 'False',
                    'False', 'True', 'False',
                    'False', 'False', 'True']
        self.assertTrue(run_barebone(commands, expected, 1))

    def test_constructors_is_nominal_instance(self):
        commands = bottom + retype + instantiate_example + is_nominal_instance + ['"return"', 'false']
        expected = ['True', 'True', 'False',
                    'False', 'True', 'False',
                    'False', 'False', 'True']
        self.assertTrue(run_barebone(commands, expected, 1))

    def test_constructors_is_structural_instance(self):
        commands = bottom + retype + instantiate_example + is_structural_instance + ['"return"', 'false']
        expected = ['True', 'False', 'True',
                    'True', 'True', 'True',
                    'True', 'False', 'True']
        self.assertTrue(run_barebone(commands, expected, 1))
